var I18nEngine = function($injector) {

	var self = this;
	var DEFAULT_LANGUAGE = "default";
	var words = {};
	var currentLanguage = null;

	this.interpolationKey = "__";

	this.setLanguage = function(language) {
		var defer = $injector.get('ZHPromise').defer();

		if (words[language] === undefined) {
			$injector.get('restEngine').requestMetaData("json/words/"+language+".json").then(function(response) {
				var langWords = response.data;
				words[language] = langWords;
				updateLanguage(language);
				defer.resolve(langWords);
			}).catch(function(error) {
				defer.reject(error);
			});
		} else {
			updateLanguage(language);
			$injector.get('eventAggregator').publish('onChangeLanguage');
			defer.resolve(words[language]);
		}

		return defer.promise;
	};

	function updateLanguage(language) {
			currentLanguage = language;
			$injector.get('localStorageEngine').setLocalVar('language', language);
			$injector.get('eventAggregator').publish('onChangeLanguage');
	}

	this.getWords = function(language) {
		return words[language];
	};

	this.i18n = function(key, variables) {
		var	languageWords = getWordsByLanguage();
		return replaceValues((languageWords[key] || key), variables);
	};

	function getWordsByLanguage() {
		return words[currentLanguage] || words[DEFAULT_LANGUAGE] || {};
	}

	function replaceValues(word, variables) {
		if (Util.isObject(variables)) {
			Object.keys(variables).forEach(function(key) {
				word = word.replace(getKeyName(key), variables[key]);
			});
		}
		return word;
	}

	function getKeyName(key) {
		return self.interpolationKey + key + self.interpolationKey;
	}

};